/**
 * Simple program to show the use of the PAPI library.
 *
 * (c) 2017,  Jens Teubner, TU Dortmund University, DBIS Group
 *
 * $Id$
 */

#include <stdio.h>
#include <stdlib.h>

#include "papi.h"

/* these are defined in papi.h */
#if 0
#define PAPI_L1_DCM  0x80000000
#define PAPI_L1_ICM  0x80000001
#endif

#define NUM_ITEMS    100000000
#define NUM_ACCESSES 10000000

int
main (int argc, char **argv)
{
    int ret;    /* used to receive return values from function calls */

    /* we're interested in L1 Data and Instruction Cache Misses */
    int events[] = { PAPI_L1_DCM , PAPI_L1_ICM };

    /* put counter measurements here */
    long long evt_vals[ sizeof (events) / sizeof (*events) ];

    /* array, to be populated with random numbers */
    int *data;

    /* allocate the array ... */
    data = malloc (NUM_ITEMS * sizeof (*data));
    if (!data)
    {
        fprintf (stderr, "Error allocating data.\n");
        exit (EXIT_FAILURE);
    }

    /* ... and populate it */
    for (unsigned int i = 0; i < NUM_ITEMS; i++)
    {
        data[i] = rand ();
    }

    printf ("Data array populated with %i elements.\n", NUM_ITEMS);

    /* start performance counters */
    if ((ret = PAPI_start_counters (events, sizeof (events) / sizeof (*events)))
            != PAPI_OK)
    {
        fprintf (stderr, "Error starting PAPI counters.\n");
        exit (EXIT_FAILURE);
    }

    /* "work"; do random probes into the array */
    int sum = 0;
    for (unsigned int i = 0; i < NUM_ACCESSES; i++)
    {
        sum += data[ rand() % NUM_ITEMS ];
    }

    /* stop performance counters, put results in evt_vals */
    if ((ret = PAPI_stop_counters (evt_vals,
                    sizeof (events) / sizeof (*events))) != PAPI_OK)
    {
        fprintf (stderr, "Error stopping PAPI counters.\n");
        exit (EXIT_FAILURE);
    }

    /* print sum, just so the compiler won't optimize our code too much */
    printf ("Counter value after %i random probes (sum was %i).\n",
            NUM_ACCESSES, sum);

    /* print counter results */
    for (unsigned int i = 0; i < sizeof (events) / sizeof (*events); i++)
        printf ("Counter %i: %20lli\n", i, evt_vals[i]);

    printf ("Done.\n");

    exit (EXIT_SUCCESS);
}
